import json
import os

class Config:
    DEFAULT_MONGODB_CONNECTION_STRING = "mongodb://localhost:27017/"

def try_get(object, key):
    """Tries to get the value of a key in an object, and politely notifies you if it cannot find it."""

    if key not in object:
        raise KeyError("Could not find " + key + " inside of " + object + "! Does it exist?")

    return object[key]


class DataConfig:
    """Holds data and filepath configuration information for data files such as state names, hashtags we wish to track,
    terms that indicate fires or disasters, etc."""

    # Folder name that data is stored in.
    DATA_FOLDER_NAME = "data"

    # Path data is stored in.
    DATA_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), DATA_FOLDER_NAME)

    # Disaster hashtag datafile location.
    DISASTER_HASHTAGS_DATA_PATH = os.path.join(DATA_PATH, "DisasterHashtags.yml")

    # Fire hashtag datafile location.
    FIRE_HASHTAGS_DATA_PATH = os.path.join(DATA_PATH, "FireHashtags.yml")

    # US major cities datafile location.
    MAJOR_CITIES_DATA_PATH = os.path.join(DATA_PATH, "MajorCities.yml")

    # US states datafile location.
    US_STATES_DATA_PATH = os.path.join(DATA_PATH, "USStates.yml")

    TWITTER_ACCOUNTS_DATA_PATH=os.path.join(DATA_PATH, "TwitterAccounts.yml")



import unittest

from twitter_fire_scraper.scraper import Scraper
from twitter_fire_scraper.twitter import TwitterAuthentication
from pymongo import MongoClient


class TestSimpleScraping(unittest.TestCase):

    def testCanScrapeTerm(self):
        """Tests that scraper can scrape one term."""

        twauth = TwitterAuthentication.autodetect_twitter_auth()

        scraper = Scraper(twitter_authentication=twauth)

        results = scraper.scrape_terms({"fire"}, count=1)

        assert('fire' in results)

        assert(isinstance(results['fire'], list))

        assert (len(results.keys()) == 1)

        assert (isinstance(results['fire'][0].text, str))

    def testCanScrapeAccount(self):
        """Tests that scraper can scrape one account."""
        twauth = TwitterAuthentication.autodetect_twitter_auth()

        scraper = Scraper(twitter_authentication=twauth)

        results = scraper.scrape_accounts({"@RedCross"}, count=1)

        assert ('@RedCross' in results)

        assert (len(results.keys()) == 1)

        assert(isinstance(results['@RedCross'], list))

        assert (isinstance(results['@RedCross'][0].text, str))

    def testCanScrapeMethod(self):
        """Tests that the Scraper's `scrape` method works."""

        twauth = TwitterAuthentication.autodetect_twitter_auth()

        scraper = Scraper(twitter_authentication=twauth)

        results = scraper.scrape(terms={"fire"}, accounts={"@RedCross"})

        assert('fire' in results.keys())
        assert('@RedCross' in results.keys())

        assert(len(results.keys()) == 2)

        assert(isinstance(results['fire'], list))
        assert(isinstance(results['@RedCross'], list))

    def testCanScrapeAndSave(self):
        """Tests if the Scraper can both scrape and save the results to a MongoDB database"""

        # Before starting, if the test database exists, remove it
        # TODO: standardize localhost string
        test_client = MongoClient()
        test_db = "testdb"
        test_client.drop_database(test_db)

        twauth = TwitterAuthentication.autodetect_twitter_auth()

        scraper = Scraper(twitter_authentication=twauth)

        results = scraper.scrape_and_save(terms={"fire"}, count=1, dbname="testdb")

        assert ('fire' in results.keys())

        assert (len(results.keys()) == 1)

        assert (isinstance(results['fire'], list))

        assert (test_client[test_db].get_collection("fire").count() == 1)
        test_client.drop_database(test_db)

    def _disabled_testLotsOfTweets(self):
        """Tests if the Scraper can retrieve 1000 tweets for one term"""
        twauth = TwitterAuthentication.autodetect_twitter_auth()

        scraper = Scraper(twitter_authentication=twauth)

        results = scraper.scrape(terms={"fire"}, count=1000)

        assert ('fire' in results.keys())

        assert (len(results.keys()) == 1)

        assert (isinstance(results['fire'], list))

        assert (len(results['fire']) == 1000)




"""
Demonstrates the ability to scrape tweets regarding chicago fires, and store them in a MongoDB database.

This does NOT use the Scraper object.
"""
# noinspection PyUnresolvedReferences
from pymongo import MongoClient
from pymongo.errors import DuplicateKeyError

import __init__

import argparse
import os

import tweepy
from tweepy import Status

import yaml
from twitter_fire_scraper.config import DataConfig, Config
from twitter_fire_scraper.twitter import TwitterAuthentication, GEOBOX_CHICAGO
from twitter_fire_scraper.util import geobox_to_geocode, status_to_url


def get_fire_search_terms():
    # Read hashtags from datafile.
    with open(DataConfig.FIRE_HASHTAGS_DATA_PATH) as f:
        fire_hashtags_data = yaml.load(f)
        return fire_hashtags_data['hashtags']


if __name__ == "__main__":

    # Prepare for command-line input.
    parser = argparse.ArgumentParser()
    parser.add_argument("-t", "--maxtweets", help="Max tweets to get.", required=False, default=10, type=int)

    # Parse arguments.
    args = parser.parse_args()

    # Maximum tweets to search for.
    MAX_TWEETS = args.maxtweets if args.maxtweets else \
        10

    # Radius to search for tweets in.
    TWEET_RADIUS = "50mi"

    # Set up twitter auth.
    twauth = TwitterAuthentication.autodetect_twitter_auth()

    api = tweepy.API(twauth.oauth_handler, wait_on_rate_limit=True, wait_on_rate_limit_notify=True)

    # Get Twitter geocode from Chicago's geobox.
    geocode = geobox_to_geocode(GEOBOX_CHICAGO, TWEET_RADIUS)

    # List of search terms to search for.
    search_terms = get_fire_search_terms()

    print(
        "Scraping the top {n} tweets that reference search terms from our '{path}' datafile inside {geocode}".format(
            n=MAX_TWEETS, path=os.path.basename(DataConfig.FIRE_HASHTAGS_DATA_PATH), geocode=geocode))

    # All tweets, sorted by search term
    all_tweets = {}

    # For all search terms,
    for search_term in search_terms:  # type: str

        # Conduct a search.
        cursor = tweepy.Cursor(api.search, q=search_term, geocode=geocode)

        # Retrieve however many tweets we want, and store that in our dictionary.
        all_tweets[search_term] = [status for status in cursor.items(MAX_TWEETS)]

        print("{n} hits for {ht}:".format(n=len(all_tweets[search_term]), ht=search_term))

        # For all statuses that we retrieved,
        for status in all_tweets[search_term][0:5]:  # type: Status
            print(" " * 4),  # Prefix with four spaces to show hierarchy.
            print("<{}> :".format(status_to_url(status)))  # URL of tweet.

            print(" " * 4),  # Status belonging to URL.
            print(status.text.encode("UTF-8"))  # Status text.
            print

    print("Final results of scraping {n} tweets each from these search terms:".format(
        n=MAX_TWEETS))

    print(", ".join(search_terms))

    # Determine unique statuses found from all keywords

    # Unique statuses. Sets can contain no duplicate elements.
    unique_status_ids = set()  # type: set[long]
    total_statuses = 0  # Total amount of statuses found.
    for keyword, statuses in all_tweets.items():

        for status in statuses:
            unique_status_ids.add(status.id)  # Add the ID as it can be enumerated in a set and is unique.
            total_statuses += 1  # 1 more status!

    # Print out (by keyword) how many hits we get
    for keyword, statuses in all_tweets.items():
        print("{keyword:20s}: {n} hits".format(keyword=keyword, n=len(statuses)))

    # Print out how many unique statuses we get
    print("{uq} unique statuses out of {tot} total statuses with {dup} duplicates.".format(
        uq=len(unique_status_ids), tot=total_statuses, dup=total_statuses - len(unique_status_ids)))

    print("Saving to MongoDB database.")

    try:
        mongoclient = MongoClient(Config.DEFAULT_MONGODB_CONNECTION_STRING)

        # Save to a table that's the same name as the file because this is a test.
        mongodb = mongoclient[os.path.splitext(os.path.basename(__file__))[0]]
    except Exception as e:
        print("Failed to save tweets to MongoDB database.")
        print(e)
        exit(1)

    saved_tweets = 0
    for keyword, statuses in all_tweets.items():
        for status in statuses:  # type: Status

            obj = status._json

            # Create unique ID that is the ID of the tweet itself.
            obj['_id'] = status.id

            try:
                mongodb[keyword].insert_one(obj)
                saved_tweets += 1
            except DuplicateKeyError as e:  # Tweet already exists.
                pass

    print("Successfully saved all tweets. Saved {} new tweets.".format(saved_tweets))



"""
This is a utility to provide a large amount of tweets to unit tests WITHOUT repeatedly incurring API hits.

This is cached in MEMORY, not to disk.

The idea here is this file gets run once and its data used repeatedly, instead of scraping the same terms again and again.

This is not strictly necessary, but will decrease API calls significantly if you don't care about reusing tweets.
"""
from typing import Dict, List

from tweepy import Status
from twitter_fire_scraper.util import geobox_to_geocode

from twitter import GEOBOX_CHICAGO
from twitter_fire_scraper.scraper import TwitterAuthentication, Scraper

from functools import lru_cache


class CachedTweets:
    scraper = Scraper(twitter_authentication=TwitterAuthentication.autodetect_twitter_auth())

    @staticmethod
    @lru_cache(maxsize=None)
    def tweets_small():
        # type: () -> Dict[str, List[Status]]
        """Return a static list of 9 tweets that is generated once and re-used throughout the module's lifetime."""
        return CachedTweets.scraper.scrape_terms({"flood", "fire", "house fire"}, count=3)

    @staticmethod
    @lru_cache(maxsize=None)
    def tweets_small_no_retweets():
        # type: () -> Dict[str, List[Status]]
        """Return a static list of 9 non-retweet tweets that is generated once and re-used throughout the module's lifetime."""
        return CachedTweets.scraper.scrape_terms({"flood", "fire", "house fire"}, count=3, include_retweets=False)

    @staticmethod
    @lru_cache(maxsize=None)
    def tweets_small_geo():
        # type: () -> Dict[str, List[Status]]
        """
        Return a static list of 9 tweets geotagged 20 miles from Chicago's center that is generated once and re-used
        throughout the module's lifetime.
        """
        return CachedTweets.scraper.scrape_terms({"flood", "fire", "house fire"}, count=3,
                                                 geocode=geobox_to_geocode(GEOBOX_CHICAGO, "20mi"))

    @staticmethod
    @lru_cache(maxsize=None)
    def tweets_medium():
        # type: () -> Dict[str, List[Status]]
        """Return a static list of 60 tweets that is generated once and re-used throughout the module's lifetime."""
        return CachedTweets.scraper.scrape_terms({"flood", "fire", "house fire"}, count=20)

    @staticmethod
    @lru_cache(maxsize=None)
    def tweets_medium_no_retweets():
        # type: () -> Dict[str, List[Status]]
        """Return a static list of 60 non-retweet tweets that is generated once and re-used throughout the module's lifetime."""
        return CachedTweets.scraper.scrape_terms({"flood", "fire", "house fire"}, count=20, include_retweets=False)

    @staticmethod
    @lru_cache(maxsize=None)
    def tweets_medium_geo():
        # type: () -> Dict[str, List[Status]]
        """
        Return a static list of 60 tweets geotagged 20 miles from Chicago's center that is generated once and re-used
        throughout the module's lifetime.
        """
        return CachedTweets.scraper.scrape_terms({"flood", "fire", "house fire"}, count=20,
                                                 geocode=geobox_to_geocode(GEOBOX_CHICAGO, "20mi"))

    @staticmethod
    @lru_cache(maxsize=None)
    def tweets_large():
        # type: () -> Dict[str, List[Status]]
        """Return a static list of 300 tweets that is generated once and re-used throughout the module's lifetime."""
        return CachedTweets.scraper.scrape_terms({"flood", "fire", "house fire"}, count=100)

    @staticmethod
    @lru_cache(maxsize=None)
    def tweets_large_geo():
        # type: () -> Dict[str, List[Status]]
        """Return a static list of 300 tweets geotagged 20 miles from Chicago's center that is generated once and
        re-used throughout the module's lifetime."""
        return CachedTweets.scraper.scrape_terms({"flood", "fire", "house fire"}, count=100,
                                                 geocode=geobox_to_geocode(GEOBOX_CHICAGO, "20mi"))



"""
Demonstrates the ability to search for fire regardless of location.
"""
# noinspection PyUnresolvedReferences
import __init__

import tweepy

from pprint import pprint

from twitter_fire_scraper.twitter import TwitterAuthentication

if __name__ == "__main__":
    # Set up twitter auth.
    twauth = TwitterAuthentication.autodetect_twitter_auth()
    api = tweepy.API(twauth.oauth_handler)

    print("Just searching for 'fire'... Probably not going to get us Chicago fire incidents.")

    for status in api.search("fire"):
        print(status.text)



import json
import os

import tweepy
from pymongo import MongoClient
from tweepy import OAuthHandler, Status

from twitter_fire_scraper.config import Config
from twitter_fire_scraper.models import Point

GEOBOX_WORLD = [Point(-180, -90), Point(180, 90)]

# From http://boundingbox.klokantech.com/
GEOBOX_CHICAGO = [Point(41.573604, -87.965887, ), Point(42.085967, -87.367663, )]


class TwitterAuthentication(object):
    """Twitter authentication object.

    This is basically just a wrapper for Twitter API keys to prevent a bunch of variables being scattered everywhere.
    """

    @staticmethod
    def autodetect_twitter_auth(auth_filepath="~/secrets.json"):
        # type: (str) -> TwitterAuthentication
        """
        Attempts to autodetect Twitter API keys from a file called 'secrets.json'.

        Using this method is inadvisable and only exists to aid our test cases.
        """
        print("WARNING: API key autodetection is inadvisable.")

        auth_filename = os.path.basename(auth_filepath)
        auth_filepath = os.path.expanduser(auth_filepath)

        if not os.path.isfile(auth_filepath):
            print("Auto-detection of {} failed. Searched this path for {}:".format(auth_filename, auth_filename))
            print(auth_filepath)

            print("Either initialize {} with API keys, or make the file located at the above path.".format(
                TwitterAuthentication.__name__))

            raise ValueError("No API keys in {} initializer".format(TwitterAuthentication.__name__))
        else:  # Path to auth file exists.
            return TwitterAuthentication.from_json(auth_filepath)

    @staticmethod
    def from_json(filepath):
        """
        Creates a TwitterAuthentication object from a JSON file.
        :param filepath: The path to the JSON file.
        :return: A TwitterAuthentication object.

        Uses the following keys:
            - consumer_key
            - consumer_secret
            - access_token
            - access_token_secret

        """
        file = open(filepath, 'r')
        json_object = json.load(file)
        file.close()

        return TwitterAuthentication(
            consumer_key=json_object['consumer_key'],
            consumer_secret=json_object['consumer_secret'],
            access_token=json_object['access_token'],
            access_token_secret=json_object['access_token_secret'],
        )

    def __init__(self, consumer_key, consumer_secret, access_token, access_token_secret):

        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.access_token = access_token
        self.access_token_secret = access_token_secret

        self.oauth_handler = OAuthHandler(self.consumer_key, self.consumer_secret)
        self.oauth_handler.set_access_token(self.access_token, self.access_token_secret)


# override tweepy.StreamListener to add logic to on_status
class SimpleFireStreamListener(tweepy.StreamListener):
    """A naive twitter stream listener that only watches for the phrase "fire".
    This is a simple demo and is best used on the command-line."""

    @staticmethod
    def is_relevant(status, verbose=False):
        # type: (Status, bool) -> bool
        """
        Tells you if a Status is relevant.
        :param status: The status.
        :param verbose: Should we print irrelevant statuses?
        :return: Whether or not the status is relevant.
        """

        text = status.text

        if 'fire' in text:
            print("Relevant:")
            return True

        if verbose:
            # Show snapshot of irrelevant tweet
            print("Not relevant: {}".format(text))

        return False

    def on_status(self, status):
        # type: (SimpleFireStreamListener, Status) -> None

        if SimpleFireStreamListener.is_relevant(status, verbose=True):
            text = status.text

            print("Relevant: " + text.encode("UTF-8"))


class MongoDBStreamListener(tweepy.StreamListener):
    """A twitter stream listener that logs flagged tweets to a MongoDB database."""

    @staticmethod
    def is_relevant(status, verbose=False):
        # type: (Status, bool) -> bool
        """
        Tells you if a Status is relevant.
        :param status: The status.
        :param verbose: Should we print irrelevant statuses?
        :return: Whether or not the status is relevant.
        """

        if 'fire' in status.text:
            return True

        return False

    def __init__(self, database_name="MongoDBStreamListener",
                 database_connection_string=Config.DEFAULT_MONGODB_CONNECTION_STRING):
        super(MongoDBStreamListener, self).__init__()

        # MongoDB client.
        self.mongoclient = MongoClient(database_connection_string)

        # MongoDB database name.
        self.mongodatabase = self.mongoclient[database_name]

        # Table to which tweets are saved.
        self.TWEETS_TABLE = database_name

    # noinspection PyUnresolvedReferences
    def on_status(self, status):
        # type: (MongoDBStreamListener, Status) -> None

        # Make text safe to print in console.
        safe_text = status.text.encode('UTF-8')

        # If the status is relevant,
        if MongoDBStreamListener.is_relevant(status):
            # Insert it into our database.
            self.mongodatabase[self.TWEETS_TABLE].insert_one(status._json)
            print("Hit: " + safe_text)
        else:
            print("Ign: " + safe_text)



"""
This is a suite of automated tests, meant to be run on various quantifiable parts of this project to ensure they work as
intended.
"""
import os
import sys
import unittest

# Append parent directory to enhance portability.
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))



name = "twitter_fire_scraper"

import twitter_fire_scraper.twitter
import twitter_fire_scraper.config
import twitter_fire_scraper.hashtags
import twitter_fire_scraper.util
import twitter_fire_scraper.models
import twitter_fire_scraper.scraper


"""
Demonstrates the ability to listen in a geographical area for a keyword tweet and save it to a MongoDB database.
"""
# noinspection PyUnresolvedReferences
import __init__

import tweepy

from twitter_fire_scraper.twitter import TwitterAuthentication, MongoDBStreamListener, GEOBOX_CHICAGO
from twitter_fire_scraper.util import geobox_from_points

if __name__ == "__main__":
    # Set up twitter auth.
    twauth = TwitterAuthentication.autodetect_twitter_auth()
    api = tweepy.API(twauth.oauth_handler)

    print("Listening for fires and logging to a MongoDB database.")
    mongolistener = MongoDBStreamListener()
    mongostream = tweepy.Stream(auth=api.auth, listener=mongolistener)

    mongostream.filter(locations=geobox_from_points(GEOBOX_CHICAGO))



import __init__

import os
import sys
import unittest

if __name__ == "__main__":
    loader = unittest.TestLoader()

    # Load current directory
    suite = loader.discover(os.path.abspath(os.path.dirname(__file__)))

    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(suite)



from __future__ import print_function
import colorama
import json
from pymongo.database import Database
from pymongo.errors import DuplicateKeyError
from tweepy import Status
from typing import Dict, List

from twitter_fire_scraper.models import Point


def dict_from_status(status):
    # type: (Status) -> dict
    """

    :param status: A Status object.
    :return: An object ready to be saved into a MongoDB database.
    """

    obj = status._json  # type: dict

    obj['_id'] = status.id

    return obj

def save_statuses_dict_to_json(status_dict, filepath="tweets.json"):
    """
    Utility function to extract tweets from a dictionary of Statuses and saves them to a csv file

    :param status_dict: A dict of {"category": {Status, Status, Status}, ...} objects
    :param filepath: Path to file where to save all tweets to
    """

    # Open file to write to
    with open(filepath, 'w') as outfile:
        json.dump(status_dict, outfile, indent=4)

    return

def save_statuses_dict_to_mongodb(status_dict, mongodb, print_on_duplicates=False):
    # type: (Dict[str, List[Status]], Database, bool) -> None
    """
    This is a utility function that saves a Dict[str, List[Status]] to a MongoDB database.

    It saves each Status to a collection of the same name as the dictionary key

    :param status_dict: A dict of {"category": {Status, Status, Status}, ...} objects.
    :param mongodb: A MongoDB Database object.
    :param print_on_duplicates Whether to log if we found a duplicate Status or not.
    """

    for category, statuses in status_dict.items():
        for status in statuses:  # type: Status

            # Create a dict from the status
            obj = dict_from_status(status)

            try:
                # Attempt to insert a status.
                mongodb[category].insert_one(obj)
            # If the status already exists,
            except DuplicateKeyError as e:
                # Error silently and continue (or print and continue)
                if print_on_duplicates:
                    print("Duplicate tweet ID {} was NOT inserted to {} collection. ".format(obj['_id'], category))
                pass


def save_single_status_to_mongodb(status, mongodb):
    # type: (Status) -> None

    # Create a dict from the status
    obj = dict_from_status(status)

    try:
        # Attempt to insert a status.
        mongodb.insert_one(obj)
    # If the status already exists,
    except DuplicateKeyError as e:
        # Error silently and continue (or print and continue)
        # if print_on_duplicates:
        print("Duplicate tweet ID {} was NOT inserted to {} collection. ".format(obj['_id'], category))
        pass


def merge_status_dict(d1, d2):
    # type: (Dict[str, List[Status]], Dict[str, List[Status]]) -> Dict[str, List[Status]]
    """
    Given two status dictionaries, merge them into one status dictionary.
    Does not modify either dictionary, and rather makes a new one.
    """
    results = dict()

    for term, statuses in d1.items():
        if term not in results:
            results[term] = list()

        for status in statuses:
            results[term].append(status)

    for term, statuses in d2.items():
        if term not in results:
            results[term] = list()

        for status in statuses:
            results[term].append(status)

    return results

def status_to_url(status):
    # type: (Status) -> str
    """Given a Status, return that status' url."""
    return "https://www.twitter.com/statuses/{id}".format(id=status.id)


def pretty_print_statuses(statuses):
    # type: (List[Status]) -> None
    for status in statuses:
        print("<{}>".format(status_to_url(status)))
        print(status.text)
        print()


def jsonify_status_dict(status_dict):
    # type: (Dict[str, List[Status]]) -> Dict[str, List[str]]
    """Take a Dict[str, List[Status]] and flatten its statuses into JSON objects.

    Example:

        flatten_status_dict({
            "icecream": [Status, Status],
            "cake": [Status]
            })

            ->

            {
                "icecream": [{"author": "@IceCreamLuvr", ...}, {"author": "@IceCreamHater", ...}],
                "cake": [{"author": "@CakeIsGod123", ...}]
            }
    """
    for term, statuses in status_dict.items():  # Only get the json of the tweet
        status_dict[term] = list([status._json for status in statuses])

    return status_dict


def flatten_status_dict(status_dict):
    # type: (Dict[str, List[Status]]) -> Dict[str, List[str]]
    """Take a Dict[str, List[Status]] and flatten its statuses into the text of the statuses.

    Example:

        flatten_status_dict({
            "icecream": [Status, Status],
            "cake": [Status]
            })

            ->

            {"icecream": ["mm icecream", "icecream sucks :("],
            "cake": ["CAKE!!!!"]}

    """
    for term, statuses in status_dict.items():  # Only print the text of the tweet
        status_dict[term] = list([status.text for status in statuses])

    return status_dict


def geobox_from_points(points):
    # type: (List[Point]) -> List[float]
    """Given a list of points, flatten them starting from y and going to x.

    This function exists because for some reason Tweepy/Twitter API likes to
    have a bounding box with flipped lat/long coordinates.

    Example:
        `flatten_points([Point(x=1,y=2), Point(x=2,y=3)])`
        -->
        `[2, 1, 3, 2]`"""
    numbers = []

    for point in points:
        numbers.append(point.y)
        numbers.append(point.x)

    return numbers


def flatten_points(points):
    # type: (List[Point]) -> List[float]
    """Given a list of points, flatten them starting from x and going to y.

    Example:
        `flatten_points([Point(x=1,y=2), Point(x=2,y=3)])`
        -->
        `[1, 2, 2, 3]`"""

    numbers = []

    for point in points:
        numbers.append(point.x)
        numbers.append(point.y)

    return numbers


def geobox_to_geocode(geobox, radius):
    # type: (List[Point, Point], str) -> str
    """Given a geobox and a radius, return a valid Twitter geocode consisting of a point and radius."""
    midpoint = geobox[0].midpoint(geobox[1])

    return "{lat},{lon},{radius}".format(
        lat=midpoint.x,
        lon=midpoint.y,
        radius=radius,
    )


def is_retweet(status):
    # type: (Status) -> bool
    """Tells you if this Status is a retweet."""
    return "RT @" in status.text


def strtobool(v):
    # type: (str) -> bool
    return v.lower() in ["yes", "true", "t", "1"]


def colorama_reset():
    # type: () -> None
    print(colorama.Fore.WHITE, end='')
    print(colorama.Back.BLACK, end='')


def colorama_highlight_red(text, keyword=None):
    # type: (str, str) -> str
    """
    Highlights text red for a terminal.
    :param text: Text.
    :param keyword: Keyword to highlight.
    :return: Highlighted text.
    """

    if keyword is None:
        keyword = text

    return text.replace(keyword, (colorama.Fore.WHITE + colorama.Back.RED +
                                  keyword +
                                  colorama.Fore.WHITE + colorama.Back.BLACK))



import unittest

# noinspection PyUnresolvedReferences
from models import Point
from util import flatten_points, geobox_to_geocode


class TestGeoLib(unittest.TestCase):
    def testPointFlattening(self):
        assert flatten_points([Point(1, 2), Point(2, 3), Point(3, 4)]) == [1, 2, 2, 3, 3, 4]

    def testGeocodeCreation(self):
        assert geobox_to_geocode([Point(-1, -1), Point(2, 2)], "20km") == "0.5,0.5,20km"



# coding=utf-8
import unittest

# noinspection PyUnresolvedReferences
import __init__
from hashtags import contains_hashtag, extract_hashtags


class TestHashtags(unittest.TestCase):

    def test_extract(self):
        assert (extract_hashtags("I love the #beach and #sand!") == {"#beach", "#sand"})

        assert (extract_hashtags(u"Pizza is so good! #~# 🍕") == set())

        assert (extract_hashtags("Man, Hashtags (#) are # nice # to # use # as # list # columns!") == set())

    def _disabled_test_unicode(self):  # Expect to fail this.
        # TODO add RTL unicode support to hashtag detection.
        assert (extract_hashtags(u"مرحبا# to the #world!") == {u'مرحبا#', '#world'})

    def test_simple_contains(self):
        assert (contains_hashtag('I enjoy #swimming!', 'swimming'))
        assert (contains_hashtag('I enjoy #swimming so much!', 'swimming'))

        assert (not (contains_hashtag("I dislike #coldswimming.", "#swimming")))
        assert (not (contains_hashtag("I dislike #coldswimming.", "#cold")))
        assert (contains_hashtag("I dislike #coldswimming.", "#coldswimming"))



import unittest

from tests.test.incur_api_hits.cached_tweets import CachedTweets


class TestNoRetweets(unittest.TestCase):

    def testNoRetweets(self):
        """Tests that scraper can successfully scrape without retweets."""

        results = CachedTweets.tweets_medium_no_retweets()

        for term, statuses in results.items():
            for status in statuses:
                assert(status.retweeted == False)
                assert(status.author.id == status.user.id)



"""
This file tests that the scraper fails correctly with invalid credentials.
"""
import unittest
from pprint import pprint

from tweepy import TweepError

from scraper import Scraper
from twitter import TwitterAuthentication
from util import flatten_status_dict

class TestInvalidCredentials(unittest.TestCase):

    def test_invalid(self):
        # This is one way to ideally initialize a Scraper object.
        scraper = Scraper(twitter_authentication=TwitterAuthentication(
            consumer_key="i4m4c0ns4m3r",
            consumer_secret="itsasecret",
            access_token="chuckecheesetokens?",
            access_token_secret="ticketmuncherdoesn'twork"
        ))

        terms = {"#pizza", "pizza", 'cat', '#cat'}

        failure = False

        try:
            results = scraper.scrape_terms(terms=terms, count=3)
        except TweepError as e:
            print("Failed successfully with bogus credentials.")
            failure = True

        if not failure:
            raise Exception("Credentials should fail to validate.")


class Point(object):
    """A 2d point.
    Has x and y."""

    def __init__(self,  # type: Point
                 x,  # type: float
                 y  # type: float
                 ):
        self.x = x
        self.y = y

    def midpoint(self, other):
        # type: (Point, Point) -> Point
        """Return a point between two points."""

        return Point(
            (self.x + other.x) / 2.0,
            (self.y + other.y) / 2.0
        )

    def flatten(self):
        # type: (Point) -> tuple[float, float]
        """Flattens a point into (x,y) coords, respectively."""
        return self.x, self.y



"""
This module is a suite of tests that are not easily test, i.e. qualitative parts of this project.

They are meant to be run by humans and inspected to make sure they do what they say they do.
"""
import os
import sys

# Append parent directory to enhance portability.
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))


"""
This file demonstrates the Scraper class' ability to scrape small amounts of tweets about a simple term or hashtag, like
'pizza', '#pizza', 'cat', and '#cat'.
"""

import __init__

from pprint import pprint

from twitter_fire_scraper.scraper import Scraper
from twitter_fire_scraper.twitter import GEOBOX_CHICAGO
from twitter_fire_scraper.util import geobox_to_geocode, flatten_status_dict

if __name__ == '__main__':
    scraper = Scraper()

    terms = {"#pizza", "pizza", 'cat', '#cat'}

    results = scraper.scrape_terms(terms=terms, count=3)
    results = flatten_status_dict(results)

    print("{} on all of twitter:".format(", ".join(terms)))
    pprint(results)

    results = scraper.scrape_terms(geocode=geobox_to_geocode(GEOBOX_CHICAGO, "20mi"), terms=terms, count=3)
    results = flatten_status_dict(results)
    print("{} in chicago:".format(", ".join(terms)))
    pprint(results)

    # scraper = Scraper()
    #
    # results = scraper.scrape_accounts(accounts=['MABASIllinois','NWSChicago'])
    #
    # pprint(results)



import csv
import os
import shutil
import tempfile
import unittest

import twitter_fire_scraper
from twitter_fire_scraper.scraper import Scraper
from twitter_fire_scraper.tests.test.incur_api_hits.cached_tweets import CachedTweets


class TestFileSaving(unittest.TestCase):

    def count_statuses(self, statuses):
        total_statuses = 0
        for keyword, statuses in statuses.items():
            for status in statuses:
                total_statuses += 1

        return total_statuses

    def setUp(self):
        # Temp folder for CSV files
        self.temp_folder = os.path.join(tempfile.gettempdir(), twitter_fire_scraper.__name__, TestFileSaving.__name__)

        # Scraper for scraping
        self.scraper = CachedTweets.scraper

        # Assorted filetype dirs
        self.csv_folder = os.path.join(self.temp_folder, 'csv')
        self.json_folder = os.path.join(self.temp_folder, 'json')

        os.makedirs(self.temp_folder, exist_ok=True)
        os.makedirs(self.csv_folder, exist_ok=True)
        os.makedirs(self.json_folder, exist_ok=True)

        print("Saved CSV files from test cases can be found at:")
        print(self.temp_folder)

    def testSaveCSVSmall(self):
        """Tests that the scraper can produce small CSV files."""
        tweets = CachedTweets.tweets_small_geo()

        tweets_csv_path = os.path.join(self.csv_folder, 'tweets_small.csv')

        self.scraper.save_statusdict_to_csv(tweets, tweets_csv_path, overwrite=True)

        total_lines = 0
        with open(tweets_csv_path, 'r', encoding='utf-16') as csv_file:
            csv_reader = csv.reader(csv_file, delimiter=Scraper.CSV_DELIMITER)
            for row in csv_reader:
                total_lines += 1

        total_statuses = self.count_statuses(tweets)

        # We should have as many tweets as there are lines in the file, plus one for the header.
        assert (total_lines == total_statuses + 1)

    def testSaveCSVLarge(self):
        """Tests that the scraper can produce large CSV files."""
        tweets = CachedTweets.tweets_large_geo()

        tweets_csv_path = os.path.join(self.csv_folder, 'tweets_large.csv')

        self.scraper.save_statusdict_to_csv(tweets, tweets_csv_path, overwrite=True)

        total_lines = 0
        with open(tweets_csv_path, 'r', encoding='utf-16') as csv_file:
            csv_reader = csv.reader(csv_file, delimiter=Scraper.CSV_DELIMITER)
            for row in csv_reader:
                total_lines += 1

        total_statuses = self.count_statuses(tweets)

        # We should have as many tweets as there are lines in the file, plus one for the header.
        assert (total_lines == (total_statuses + 1))



"""
This file demonstrates the Scraper class' ability to scrape small amounts of tweets from a list of accounts.
"""

import __init__

from pprint import pprint

import yaml

from twitter_fire_scraper.config import DataConfig
from twitter_fire_scraper.scraper import Scraper

if __name__ == '__main__':
    scraper = Scraper()

    with open(DataConfig.TWITTER_ACCOUNTS_DATA_PATH, errors='ignore') as f:
        accountsYml = yaml.load(f)

    # First 5 accounts from Twitter Account list.
    accounts = accountsYml['accounts'][0:5]

    # Amount to get.
    count = 3

    print("Scraping top {} tweets from these accounts:".format(count))
    print(", ".join(["@" + account for account in accounts]))

    results = scraper.scrape_accounts(accounts=accounts, count=count)
    pprint(results)



"""
Demonstrates the ability to listen in a geo-location for the keyword 'fire'.
"""
# noinspection PyUnresolvedReferences
import __init__

import time

import tweepy

from twitter_fire_scraper.twitter import SimpleFireStreamListener, TwitterAuthentication, GEOBOX_CHICAGO
from twitter_fire_scraper.util import geobox_from_points

if __name__ == "__main__":

    # Set up twitter auth.
    twauth = TwitterAuthentication.autodetect_twitter_auth()
    api = tweepy.API(twauth.oauth_handler)

    print("Using our simple 'fire' stream listener, let's see what Chicago geotagged tweets have 'fire' in them.")

    simpleFireStreamListener = SimpleFireStreamListener()
    simpleFireStream = tweepy.Stream(auth=api.auth, listener=simpleFireStreamListener)

    print("Streaming for 60 seconds.")
    simpleFireStream.filter(locations=geobox_from_points(GEOBOX_CHICAGO), is_async=True)
    try:
        time.sleep(60)
    except KeyboardInterrupt:
        simpleFireStream.disconnect()
    finally:
        simpleFireStream.disconnect()



import setuptools

with open("../README.md", 'r') as f:
    long_description = f.read()

with open("VERSION", 'r') as f:
    version = f.read()

setuptools.setup(
    name="twitter-fire-scraper",
    version=version,

    author="Henry Post",
    author_email="HenryFBP@gmail.com",

    description="A tool to scrape data about fires from Twitter.",
    long_description=long_description,
    long_description_content_type="text/markdown",

    url="https://github.com/raaraa/IPRO497-Analytics-Team/tree/master/coding/twitter-fire-scraper",

    package_data = {
        'twitter-fire-scraper': [
            'data/*.yml',
            'templates/*.html'
        ]
    },

    packages=setuptools.find_packages(),
    install_requires=[
        "click",
        "itsdangerous",
        "nltk",
        "oauthlib",
        "requests",
        "requests-oauthlib",
        "six",
        "textblob",
        "tweepy",
        "Flask",
        "Jinja2",
        "MarkupSafe",
        "Werkzeug",
        "tmdbsimple",
        "colorama",
        "pymongo",
        "typing",
        "pyyaml",
    ],

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
)



"""
A file that contains utilities to detect if a hashtag refers to a disaster, as well as disaster hashtag-generating
utilities.
"""
import re
from typing import Union


def contains_hashtag(text, hashtag):
    # type: (str, str) -> bool
    """Tells you if text contains a hashtag.

    :param text: Text to search in.
    :param hashtag: Hashtag to search for.
    :return: If the hashtag exists in the text.
    """

    return ensure_hashtag(hashtag) in extract_hashtags(text)


REGEX_ALPHABETIC_HASHTAG = "#\\w+"


def ensure_hashtag(text):
    # type: (str) -> str
    """Ensure that `text` begins with '#'."""

    if len(text) is 0:  # Empty string
        return '#' + text

    if text[0] == '#':  # Starts with '#'
        return text

    return '#' + text  #


def extract_hashtags(text):
    # type: (Union[str, unicode]) -> set[str]
    """Given text, will return all hashtags present in the text.
    This assumes hashtags are alphabetic.

    TODO test non-western chars like مرحبا, 你好, or नमस्ते.
    """
    matches = re.findall(REGEX_ALPHABETIC_HASHTAG, text)
    return set(matches)



import csv
import os
from _csv import writer
from typing import Dict, List, Set

import tweepy
from tweepy import Status

from twitter_fire_scraper.twitter import TwitterAuthentication
from twitter_fire_scraper.util import merge_status_dict, save_statuses_dict_to_mongodb
from pymongo import MongoClient


class Scraper:
    CSV_DELIMITER = '\t'

    def __init__(self, twitter_authentication=None):
        # type: (Scraper, TwitterAuthentication) -> None

        # They did not pass in any authentication. Attempt to auto-detect it.
        if not twitter_authentication:
            self.twitter_authentication = TwitterAuthentication.autodetect_twitter_auth()
        else:  # They passed us a TwitterAuthentication object
            self.twitter_authentication = twitter_authentication

        # Tweepy API object. Can make API calls.
        self.api = tweepy.API(self.twitter_authentication.oauth_handler,
                              wait_on_rate_limit=True, wait_on_rate_limit_notify=True)

        # Default amount of tweets to retrieve.
        self.default_count = 10

        # Default accounts to scrape.
        # self.accounts = accounts in data/TwitterAccounts.yml

    def scrape_terms(self, terms, count=None, geocode=None, include_retweets=True):
        # type: (Scraper, Set[str], int, str) -> Dict[str, List[Status]]
        """
        Term-scraping method. Can scrape a set of terms.

        A term is either a hashtag or a piece of text.

        :param geocode: Geographical area to search in. Can be blank.
        :param terms:  List of terms to search for.
        :param count: Maximum tweets to return per search term.
        :param include_retweets: Should retweets be included?
        :return: A dictionary containing {'search-term': List[Status]} pairs.
        """

        if not count:
            count = self.default_count

        results = {}  # type: Dict[str, List[Status]]

        # For each search term,
        for search_term in terms:

            query = search_term

            if not include_retweets:
                query += ' -filter:retweets'
                query += ' -filter:nativeretweets'

            # Make a cursor that can get tweets.
            cursor = tweepy.Cursor(self.api.search, q=query, geocode=geocode, tweet_mode='extended')

            # For each result of a search term,
            for status in cursor.items(count):  # type: Status

                if search_term not in results:
                    results[search_term] = list()

                # Add the status to a particular search term.
                results[search_term].append(status)

        return results

    def scrape_accounts(self, accounts, count=None):
        # type: (Scraper, Set[str], int) -> Dict[str, List[Status]]
        """
        Account-scraping method. Can scrape a set of accounts.

        An account is either a screenname or @screenname, i.e. both 'Dude123' and '@Dude123' are valid.

        :param accounts: List of accounts to search in.
        :param count: Maximum tweets to return per account.
        :return: A dictionary containing {'@Dude123': List[Status]} pairs.
        """

        if not count:
            count = self.default_count

        results = dict()

        for account in accounts:
            statuses = self.api.user_timeline(screen_name=account, count=count)

            if account not in results:
                results[account] = list()

            for status in statuses:
                results[account].append(status)

        return results

    def scrape(self, terms=None, accounts=None, count=None, geocode=None, include_retweets=True):
        # type: (Scraper, Set[str], Set[str], int, str) -> Dict[str, List[Status]]
        """
        General-purpose scraping method. Can scrape search terms, and accounts.

        :param geocode: Geographical area to search in. Can be blank.
        :param terms:  List of terms to search for.
        :param accounts: List of account names to search.
        :param count: Maximum tweets to return per search term.
        :param include_retweets: Should retweets be included?
        :return: A dictionary containing {'search-term': List[Status]} pairs.

        Examples:
            >>> self.scrape(geocode="41.8297855,-87.666775,50mi", terms={"pizza", "waffles"}, count=3)
            {'pizza': {Status, Status, Status},
            'waffles': {Status, Status}}
        """
        if not count:
            count = self.default_count

        if (not terms) and (not accounts):
            raise ValueError("No terms or accounts specified.")

        results = dict()

        if terms:
            terms_results = self.scrape_terms(terms=terms, count=count, geocode=geocode,
                                              include_retweets=include_retweets)

            results = merge_status_dict(results, terms_results)

        if accounts:
            accounts_results = self.scrape_accounts(accounts=accounts, count=count)

            results = merge_status_dict(results, accounts_results)

        return results

    def scrape_and_save(self, terms=None, accounts=None, count=None, geocode=None, dbname='scraper_tweets'):
        # type: (Scraper, Set[str], Set[str], int, str, str) -> Dict[str, List[Status]]

        # First, retrieve search results via scrape
        results = self.scrape(terms=terms, accounts=accounts, geocode=geocode, count=count)

        # Establish connection to the host
        client = MongoClient()

        # Access database where the scraped tweets will be saved
        db = client[dbname]

        # Save all tweets to the database
        save_statuses_dict_to_mongodb(results, db)

        return results

    def save_statusdict_to_csv(self, statusdict, filepath, overwrite=False):
        # type: (Scraper, Dict[str, List[Status]], str, bool) -> str
        """Save a status dict to a CSV file.
        :param statusdict A {str: [Status, Status, ...]} dictionary.
        :param filepath A path to the file to output to."""

        dirname = os.path.dirname(filepath)

        if not os.path.exists(dirname):
            raise NotADirectoryError("Directory {} does not exist!".format(dirname))

        if os.path.isfile(filepath):
            if not overwrite:
                raise FileExistsError("File at '{}' already exists!".format(filepath))

        fieldnames = [
            'category', 'tweet_id', 'text', 'date', 'retweet_count',

            'geo',

            'coordinates',

            'place_id', 'place_centroid', 'place_country', 'place_country_code', 'place_full_name',
            'place_name', 'place_type'
        ]

        with open(filepath, 'w', encoding='utf-16', newline='') as file:
            fileWriter = csv.DictWriter(file, delimiter=Scraper.CSV_DELIMITER, quotechar='"', quoting=csv.QUOTE_ALL,
                                        fieldnames=fieldnames)

            fileWriter.writeheader()

            for keyword, statuses in statusdict.items():

                for status in statuses:  # type: Status

                    data = {
                        "category": keyword,
                        "tweet_id": status.id,
                        "text": status.text,
                        "date": status.created_at,
                        'retweet_count': status.retweet_count,
                    }

                    if status.geo:
                        data.update({
                            "geo": ','.join(str(x) for x in status.geo['coordinates']),
                        })

                    if status.coordinates:
                        data.update({
                            'coordinates': ','.join(str(x) for x in status.coordinates['coordinates']),
                        })

                    # If the status has a place, then add its data!
                    if status.place:
                        data.update({
                            'place_id': status.place.id,
                            # We have to use the API for this one to look it up.
                            'place_centroid': ','.join(str(x) for x in self.api.geo_id(status.place.id).centroid),
                            'place_country': status.place.country,
                            'place_country_code': status.place.country_code,
                            'place_full_name': status.place.full_name,
                            'place_name': status.place.name,
                            'place_type': status.place.place_type,
                        })

                    # Write all the data we've collected so far.
                    fileWriter.writerow(data)

        return filepath



import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction.text import TfidfTransformer
from sklearn.svm import LinearSVC

fire = pd.read_csv("toy_dataset1.csv")

X_train, X_test, y_train, y_test = train_test_split(fire['Tweet'],fire['label'],test_size=0.3)

count_vect = CountVectorizer()
X_train_counts = count_vect.fit_transform(X_train)
tfidf_transformer = TfidfTransformer()
X_train_tfidf = tfidf_transformer.fit_transform(X_train_counts)
clf = LinearSVC().fit(X_train_tfidf, y_train)

# print(clf.predict(count_vect.transform(["text_here"])))

def classifier(text):
    result = clf.predict(count_vect.transform([text]))
    # print(result)
    return result